@@ -1,7 +1,8 @@
 #  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Copyright (c) 2008-2022
+#  National Technology and Engineering Solutions of Sandia, LLC
 #  Under the terms of Contract DE-NA0003525 with National Technology and
 #  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain
 #  rights in this software.
@@ -281,7 +282,7 @@ class ExternalPyomoModel(ExternalGreyBoxModel):
         input_vars = self.input_vars
 
         for var, val in zip(input_vars, input_values):
-            var.set_value(val)
+            var.set_value(val, skip_validation=True)
 
         vector_scc_idx = 0
         for block, inputs in self._scc_list:
@@ -325,7 +326,7 @@ class ExternalPyomoModel(ExternalGreyBoxModel):
                     new_primals = nlp.get_primals()
                     assert len(new_primals) == len(variables)
                     for var, val in zip(variables, new_primals):
-                        var.set_value(val)
+                        var.set_value(val, skip_validation=True)
 
                 else:
                     # Use a Pyomo solver to solve this strongly connected
