@@ -1,7 +1,8 @@
 #  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Copyright (c) 2008-2022
+#  National Technology and Engineering Solutions of Sandia, LLC
 #  Under the terms of Contract DE-NA0003525 with National Technology and
 #  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain
 #  rights in this software.
@@ -21,6 +22,7 @@ from pyomo.common.log import is_debug_set
 from pyomo.common.modeling import NOTSET
 from pyomo.common.timing import ConstructionTimer
 from pyomo.core.base.component import ComponentData, ModelComponentFactory
+from pyomo.core.base.global_set import UnindexedComponent_index
 from pyomo.core.base.indexed_component import (
     IndexedComponent, UnindexedComponent_set, IndexedComponent_NDArrayMixin
 )
@@ -135,6 +137,7 @@ class _ParamData(ComponentData, NumericValue):
         # the base ComponentData constructor.
         #
         self._component = weakref_ref(component)
+        self._index = NOTSET
         #
         # The following is equivalent to calling the
         # base NumericValue constructor.
@@ -322,7 +325,7 @@ class Param(IndexedComponent, IndexedComponent_NDArrayMixin):
         """
         if self._default_val is Param.NoValue:
             return len(self._data)
-        return len(self._index)
+        return len(self._index_set)
 
     def __contains__(self, idx):
         """
@@ -331,7 +334,7 @@ class Param(IndexedComponent, IndexedComponent_NDArrayMixin):
         """
         if self._default_val is Param.NoValue:
             return idx in self._data
-        return idx in self._index
+        return idx in self._index_set
 
     # We do not need to override keys(), as the __len__ override will
     # cause the base class keys() to correctly correctly handle default
@@ -457,7 +460,7 @@ class Param(IndexedComponent, IndexedComponent_NDArrayMixin):
                 for index, new_value in new_values.items():
                     self[index] = new_value
             else:
-                for index in self._index:
+                for index in self._index_set:
                     self[index] = new_values
             return
         #
@@ -478,14 +481,14 @@ class Param(IndexedComponent, IndexedComponent_NDArrayMixin):
                 # For scalars, we will choose an approach based on
                 # how "dense" the Param is
                 if not self._data: # empty
-                    for index in self._index:
+                    for index in self._index_set:
                         p = self._data[index] = _ParamData(self)
                         p._value = new_values
-                elif len(self._data) == len(self._index):
-                    for index in self._index:
+                elif len(self._data) == len(self._index_set):
+                    for index in self._index_set:
                         self._data[index]._value = new_values
                 else:
-                    for index in self._index:
+                    for index in self._index_set:
                         if index not in self._data:
                             self._data[index] = _ParamData(self)
                         self._data[index]._value = new_values
@@ -552,6 +555,7 @@ class Param(IndexedComponent, IndexedComponent_NDArrayMixin):
                     ans = self._data[index] = _ParamData(self)
                 else:
                     ans = self._data[index] = self
+                ans._index = index
                 return ans
             if self.is_indexed():
                 idx_str = '%s[%s]' % (self.name, index,)
@@ -677,10 +681,12 @@ class Param(IndexedComponent, IndexedComponent_NDArrayMixin):
             if index is None and not self.is_indexed():
                 self._data[None] = self
                 self.set_value(value, index)
+                self._index = UnindexedComponent_index
                 return self
             elif self._mutable:
                 obj = self._data[index] = _ParamData(self)
                 obj.set_value(value, index)
+                obj._index = index
                 return obj
             else:
                 self._data[index] = value
@@ -792,7 +798,7 @@ class Param(IndexedComponent, IndexedComponent_NDArrayMixin):
             self._constructed = True
 
             # populate all other indices with default data
-            # (avoids calling _set_contains on self._index at runtime)
+            # (avoids calling _set_contains on self._index_set at runtime)
             if self._dense_initialize:
                 self.to_dense_data()
         finally:
@@ -814,7 +820,7 @@ class Param(IndexedComponent, IndexedComponent_NDArrayMixin):
             dataGen = lambda k, v: [ v, ]
         headers = [
             ("Size", len(self)),
-            ("Index", self._index if self.is_indexed() else None),
+            ("Index", self._index_set if self.is_indexed() else None),
             ("Domain", self.domain.name),
             ("Default", default),
             ("Mutable", self._mutable),
@@ -833,6 +839,7 @@ class ScalarParam(_ParamData, Param):
     def __init__(self, *args, **kwds):
         Param.__init__(self, *args, **kwds)
         _ParamData.__init__(self, component=self)
+        self._index = UnindexedComponent_index
 
     #
     # Since this class derives from Component and Component.__getstate__
